* A program to implement the rank test in linear IV models by Chen and Fang (2019)
* Improved inference on the rank of a matrix by Chen and Fang (2019)
* Edited on 12/01/2018 by Chen and Fang (2019)
* Test for full rank with known r_0 = k-1

program define bootranktest, rclass sortpreserve
        version 14.0
		
		* Getting variable lists from the parentheses
		GetVarlist `0'
		local varlist1 `s(varlist)' // Get the first variable list
		local 0 `"`s(rest)'"'
		sret clear
		GetVarlist `0'
		local varlist2 `s(varlist)' // Get the second variable list
		local 0 `"`s(rest)'"'
		sret clear
		tsunab varlist1 : `varlist1'
	    local m : word count `varlist1'
	    tsunab varlist2 : `varlist2'
	    local k : word count `varlist2'
		
        syntax [if] [in]
		marksample touse
		
		tempname st pvalue
		
		// First step
		quietly ranktest (`varlist1') (`varlist2'), robust
		
		
		// Second step
		mata: bootstrapcv("`varlist1'", "`varlist2'", "`touse'",`k'-1, "r(rkmatrix)", "`st'", "`pvalue'")
		return scalar Teststat = `st'
		return scalar Pvalue = `pvalue'
		return local cmd "Bootstrapped Rank Test"
end

* Program for getting two varlists
program define GetVarlist, sclass 
    version 14.0
	sret clear
	gettoken open 0 : 0, parse("(") 
	if `"`open'"' != "(" {
		error 198
	}
	gettoken next 0 : 0, parse(")")
	while `"`next'"' != ")" {
		if `"`next'"'=="" { 
			error 198
		}
		local list `list'`next'
		gettoken next 0 : 0, parse(")")
	}
	sret local rest `"`0'"'
	tokenize `list'
	local 0 `*'
	sret local varlist "`0'"
end

* Mata function for calculating the critical value by bootstrapping
mata:  		
void bootstrapcv(string scalar varlist1, string scalar varlist2, string scalar touse,
              real scalar r, string scalar KPmatrix, string scalar Sname, string scalar Pname)
{
    real matrix Z, X, U, V, U0, V0, Zstar, Xstar, Mstar 
    real colvector L, S, Dstar, ind, Lstar, Sstar, KPM, pvalue1, tempindex
    real scalar  n, m, k, ZXhat, stat, B, i, ZXhatstar, r0, beta
	
	beta = 0.05/15
	KPM = st_matrix(KPmatrix)
	pvalue1 = KPM[.,3]
	
	
	if (max(pvalue1)<beta)
	{
	stat = 100000
	pvalue = max(pvalue1)
	}
	
	
	else
	{
	tempindex = selectindex(pvalue1 :>= beta)
	r0 = tempindex[1]-1
	Z = st_data(., varlist1, touse)
	X = st_data(., varlist2, touse)
    n = rows(Z)
	m = cols(Z)
	k = cols(X)
    
    ZXhat = quadcross(Z, X)/n
	U = J(m,m,0)
	V = J(k,k,0)
	L = J(k,1,0)
    fullsvd(ZXhat, U, L, V)
	U0 = U[.,r0+1::m]
	V0 = V'[.,r0+1::k]
	
	S = L[|r+1\k|]
    stat = n*S'*S
   
	
	B = 500
	Dstar = J(B,1,0)
	for(i=1;i<=500;i++)
	{
	//rseed(i^2+5)
	ind = runiformint(n,1,1,n)
	Zstar = Z[ind,.]
	Xstar = X[ind,.]
	ZXhatstar = quadcross(Zstar, Xstar)/n
	Mstar = sqrt(n):*(ZXhatstar - ZXhat)
	Lstar = svdsv(U0'*Mstar*V0)
	Sstar = Lstar[|r-r0+1\k-r0|]
    Dstar[i] = Sstar'*Sstar
	}

	pvalue = mean(Dstar:>stat)
	}
	
	st_numscalar(Sname, stat)
	st_numscalar(Pname, pvalue)
}
end


