function [JAChat] =JACPQA(DX,DY,Ahat)
% The Jacobian matrix of the third moment at the estimate
% In "Improved Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019

% Inputs: 
% DX: N*dx the data of X 
% DY: N*dy the data of Y
% Ahat: dx*dy the method of moment estimator of A
% Outputs:
% Ahat: [dx*dy]*[Kx+Ky-2+dx*dy] Jacobian matrix of the moment at the estiamte

% Idea: Use the numerical differentiation approach

% Prepare for optimization
[~,dx] = size(DX);
[~,dy] = size(DY);
[X,~,IXC] = unique(DX,'rows'); % Kx*dx the support of X, i.e., Kx mass points 
[Y,~,IYC] = unique(DY,'rows'); % Ky*dy the support of Y, i..e, Ky mass points
MX = tabulate(IXC);
MY = tabulate(IYC);
Phat = MX(:,3)/100; % Kx*1 the pmf vector of DX
Qhat = MY(:,3)/100; % Ky*1 the pmf vector of DY
Kx = length(X);
Ky = length(Y);

% Numerical approach
SigmaXYvec = @(A,P,Q) reshape(X'*OptimalPI(A,X,Y,P,Q)*Y,dx*dy,1); % vectorizaton of Population moment
SigmaXYvechat = SigmaXYvec(Ahat,Phat,Qhat);
epp = 1e-6;
J1 = zeros(dx*dy,Kx-1);
for i = 1:Kx-1
    tempt = zeros(Kx,1);
    tempt(i) = epp;
    tempt(end) = -epp;
    J1(:,i) = (SigmaXYvec(Ahat,Phat+tempt,Qhat) - SigmaXYvechat)/epp;
end
J2 = zeros(dx*dy,Ky-1);
for i = 1:Ky-1
    tempt = zeros(Ky,1);
    tempt(i) = epp;
    tempt(end) = -epp;
    J2(:,i) = (SigmaXYvec(Ahat,Phat,Qhat+tempt) - SigmaXYvechat)/epp;
end
J3 = zeros(dx*dy,dx*dy);
for i = 1:dx*dy
    tempt = zeros(dx*dy,1);
    tempt(i) = epp;
    J3(:,i) = (SigmaXYvec(Ahat+reshape(tempt,dx,dy),Phat,Qhat) - SigmaXYvechat)/epp;
end
JAChat = [J1,J2,J3];




