function [PI] = OptimalPI(A,X,Y,P,Q)
% Solving for the optimal matching probability mass function
% In "Improved Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019

% Inputs: 
% A: dx*dy affinity matrix
% X: Kx*dx the support of X, i.e., Kx mass points 
% Y: Ky*dy the support of Y, i..e, Ky mass points 
% P: Kx*1 the pmf vector of X
% Q: Ky*1 the pmf vector of Y
% Outputs:
%Pi: Kx*Ky the optimal matching probability mass function over X*Y

%Idea: Use the iterative projection fitting procedure
%This one is much faster than the below one
maxIter = 1000;
Kx = length(P);
Ky = length(Q);
K = exp(X*A*Y');
a0 = P;
ak = a0;% initial value
for j = 1 : maxIter
    bk = Q./(K'*ak/Kx);
    ak = P./(K*bk/Ky);
end
tempt = K.*(ak*bk');
PI = tempt/(sum(sum(tempt)));

% %Idea: Plug in the contraints to the objective function and solve for uncontrained maximization problem
% % Prepare for optimization
% Kx = length(P);
% Ky = length(Q);
% objf1 = @(PI1) -sum(sum((X(1:Kx-1,:)*A*Y(1:Ky-1,:)'-log(PI1)).*PI1)) - sum((X(1:Kx-1,:)*A*Y(Ky,:)'-log(P(1:Kx-1) - sum(PI1,2))).*(P(1:Kx-1) - sum(PI1,2)))...
%      - sum(((X(Kx,:)*A*Y(1:Ky-1,:)')'-log(Q(1:Ky-1) - sum(PI1,1)')).*(Q(1:Ky-1) - sum(PI1,1)')) - (X(Kx,:)*A*Y(Ky,:)'- log(sum(sum(PI1))+P(Kx)+Q(Ky)-1))*(sum(sum(PI1))+P(Kx)+Q(Ky)-1);
% 
% % Paramters for Optimization
% startPI1 = P(1:Kx-1)*Q(1:Ky-1)';
% warning off all
% opts = optimset('Display','off');
% problem = createOptimProblem('fmincon', ...
%                               'objective', @(PI1) objf1(PI1), ...
%                               'x0', startPI1, ...
%                               'options', opts);
%                           
% % Optimization
% gs = GlobalSearch('Display','off'); 
% [PI1opt1,temp1,~,~,~] = run(gs,problem);
% [PI1opt2,temp2] = fminunc(@(PI1) objf1(PI1),startPI1,opts);
% PI1opt = PI1opt1*(temp1<temp2)+PI1opt2*(temp1>=temp2);
% PI = [PI1opt,P(1:Kx-1) - sum(PI1opt,2); Q(1:Ky-1)' - sum(PI1opt,1),sum(sum(PI1opt))+P(Kx)+Q(Ky)-1]; 

