% Table 6: Empirical results
% In "Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
%% Prepare for the data
% Load the data
load('data_all.mat');
% Variables
%%%%%%%%%%%%%%%%%%%%(Change this to obtain different results)%%%%%%%%%%%%%%%%%%%%%%%
Num = [1,3,10];%---------------------------------------------------------------------------------------------------------------------------------------------------------------[Change the value for new seeting]
% Num = [1,3,5,10];
% Num = [1,3,6,10];
% Num = [1,3,7,10];
% Num = [1,3,8,10];
% Num = [1,3,9,10];
% Num = [1,3,5,6,10];
% Num = [1,3,5,9,10];
% Num = [1,3,6,9,10];
DX = data_all(1:end,Num);
DY = data_all(1:end,10 + Num);
[N,dx] = size(DX);
[~,dy] = size(DY);
clear data_all;

% Discretize the data
DX(:,2) = 1*(DX(:,2)<18.5)+2*(DX(:,2)>=18.5).*(DX(:,2)<24.99)+3*(DX(:,2)>=24.99);
DY(:,2) = 1*(DY(:,2)<18.5)+2*(DY(:,2)>=18.5).*(DY(:,2)<24.99)+3*(DY(:,2)>=24.99);
threshnumX = quantile(DX(:,3:end),[0.25,0.75]);
threshnumY = quantile(DY(:,3:end),[0.25,0.75]);
if size(threshnumX,1)==1
    thrshx1 = threshnumX(1);thrshx2 = threshnumX(2);
    thrshy1 = threshnumY(1);thrshy2 = threshnumY(2);
elseif size(threshnumX,2)>1
    thrshx1 = threshnumX(1,:);thrshx2 = threshnumX(2,:);
    thrshy1 = threshnumY(1,:);thrshy2 = threshnumY(2,:);
else
end
DX(:,3:end) = 1*(DX(:,3:end)<kron(ones(N,1),thrshx1))+2*(DX(:,3:end)>=kron(ones(N,1),thrshx1)).*(DX(:,3:end)<kron(ones(N,1),thrshx2))+3*(DX(:,3:end)>=kron(ones(N,1),thrshx2));
DY(:,3:end) = 1*(DY(:,3:end)<kron(ones(N,1),thrshy1))+2*(DY(:,3:end)>=kron(ones(N,1),thrshy1)).*(DY(:,3:end)<kron(ones(N,1),thrshy2))+3*(DY(:,3:end)>=kron(ones(N,1),thrshy2));

% Recenter the variables to have zero means
mX = mean(DX);
mY = mean(DY);
DX = DX-ones(N,dx)*diag(mX);
DY = DY-ones(N,dy)*diag(mY);

%% Estimation
% The MM estimator based on matching the cross covariance
[Ahat] = MMCROSSXY(DX,DY);
SscaleDX=diag(sqrt(diag(cov(DX))));
SscaleDY=diag(sqrt(diag(cov(DY)))); 
[Ahat1] = SscaleDX*Ahat*SscaleDY;
[U,S,V] = svd(Ahat1);

%% To test for rank inequality: multiple matching and full matching, and determine the true rank --- Chen and Fang (2018)'s bootstrap method
% Bootstrap the data
B = 1000;
WLLA1 = zeros(B,dy);
WLLA2 = zeros(B,dy);
WLLN1 = zeros(B,dy);
WLLN2 = zeros(B,dy);
WLLATwo = zeros(B,dy);
M = zeros(dx,dy,B);
parfor b = 1:B
    rng(5^2+5+10*b)
    ind = unidrnd(N*ones(N,1));
    DXstar = DX(ind,:);
    DYstar = DY(ind,:);
    mXs=mean(DXstar);
    mYs=mean(DYstar);
    DXstar=DXstar-ones(N,dx)*diag(mXs);
    DYstar=DYstar-ones(N,dy)*diag(mYs);
    [Ahatstar] = MMCROSSXY(DXstar,DYstar);
    SscaleDXstar=diag(sqrt(diag(cov(DXstar))));
    SscaleDYstar=diag(sqrt(diag(cov(DYstar))));
    [Ahat1star] = SscaleDXstar*Ahatstar*SscaleDYstar;
    M(:,:,b) = sqrt(N)*(Ahat1star-Ahat1);
end

% The method using tuning paramters
tempt = sort(svd(Ahat1).^2);
%%%%%%%%%%%%%%%%%%%%(Change this to obtain different results)%%%%%%%%%%%%%%%%%%%%%%%
kappan = [N^(-1/4);N^(-1/3)];
rzerohat = [sum(tempt>kappan(1)^2);sum(tempt>kappan(2)^2)];% estimate rzero consistently using tuning parameter
TestStat = zeros(dy,1);
PV1 = zeros(dy,5); % the last column is for the two-step procedure
for j = 1:dy % implement the test
    r = j-1;
    TestStat(j) = sum(tempt(1:min(dx,dy)-r)); % compute the test statistic for real data  
    r_hat = [min(r,rzerohat(1));min(r,rzerohat(2))];% estimate rzero consistently under the null
    U2hat1 = U(:,r_hat(1)+1:end); % for the first tuning paramter
    V2hat1 = V(:,r_hat(1)+1:end);
    for b = 1:B
        tempt5 = U2hat1'*M(:,:,b)*V2hat1;
        tempt11 = sort(svd(tempt5).^2);
        WLLA1(b,j) = sum(tempt11(1:min(dx,dy)-r)); % analytical method
        tempt6 = sort(svd(Ahat1+kappan(1)*M(:,:,b)).^2);
        WLLN1(b,j) = (sum(tempt6(1:min(dx,dy)-r))-TestStat(j))/(kappan(1)^2); % numerical method
    end
    PV1(j,1:2) = mean(N*TestStat(j)<[WLLA1(:,j),WLLN1(:,j)]);% calculate p values
    U2hat2 = U(:,r_hat(2)+1:end);% for the second tuning paramter
    V2hat2 = V(:,r_hat(2)+1:end);
    for b = 1:B
        tempt3 = U2hat2'*M(:,:,b)*V2hat2;
        tempt33 = sort(svd(tempt3).^2);
        WLLA2(b,j) = sum(tempt33(1:min(dx,dy)-r));
        tempt4 = sort(svd(Ahat1+kappan(2)*M(:,:,b)).^2);
        WLLN2(b,j) = (sum(tempt4(1:min(dx,dy)-r))-TestStat(j))/(kappan(2)^2);
    end
    PV1(j,3:4) = mean(N*TestStat(j)<[WLLA2(:,j),WLLN2(:,j)]);
end

% The two-step procedure
[~,VARA1] = VarAhat(DX,DY,Ahat);
RankKP = zeros(dy,1);
PV = zeros(dy+1,1);
PV(dy+1) = 1;
for q=1:dy
    tempt5 = kron(V(:,q:end),U(:,q:end));
    tempt6 = reshape(S(q:end,q:end),(dx-q+1)*(dy-q+1),1);
    RankKP(q) = N*tempt6'*pinv(tempt5'*VARA1*tempt5)*tempt6;
    PV(q) = 1-cdf('chi2',RankKP(q),(dx-q+1)*(dy-q+1));
end
%%%%%%%%%%%%%%%%%%%%(Change this to obtain different results)%%%%%%%%%%%%%%%%%%%%%%%
rzerohat1 = find(PV>0.005, 1)-1; % estimate for the true rank with 0.005 level in the first step
if rzerohat1==dy
    PV1(:,5) = PV(1:end-1);
elseif rzerohat1<dy
    if rzerohat1>0
        PV1(1:rzerohat1,5)= PV(1:rzerohat1);
    else
    end
    for j = rzerohat1+1:dy
        r = j-1;
        U2hatTwo = U(:,rzerohat1+1:end);
        V2hatTwo = V(:,rzerohat1+1:end);
        for b = 1:B
            tempt1Two = U2hatTwo'*M(:,:,b)*V2hatTwo;
            tempt11Two = sort(svd(tempt1Two).^2);
            WLLATwo(b,j) = sum(tempt11Two(1:min(dx,dy)-r)); % analytical method in the second step
        end
        PV1(j,5) = mean(N*TestStat(j)<WLLATwo(:,j));
    end
else
end
save result1.mat Num Ahat Ahat1 RankKP TestStat M rzerohat rzerohat1 WLLA1 WLLA2 WLLN1 WLLN2 WLLATwo PV PV1%-------------------------------------------------------------------------[Change the value for new seeting]

%% To test for rank inequality: multiple matching and full matching, and determine the true rank --- The methods based on Kleibergen and Paap (2006)'s test
% Read the result from the first step of the two-step procedure above
toc