% Figure 2: Conservativeness of the KP-M test. The number of Monte Carlo
% simulations is 10,000, ...
% In "Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% X_{i} = W_{i}*Pi_{0} + u_{i}
% X_{i} is 1*k, W_{i} is 1*m and U_{i} is 1*k
% Pi is m*k, m>=k
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Testing H_{0}: Rank(Pi_{0})<=r v.s. H_{1}: Rank(Pi_{0})>r
% To show our test is more powerful.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n = 1000;
r = 5;
%%%%%%%%%%%%%%%%%%%%(Change this to obtain different curves)%%%%%%%%%%%%%%%%%%%%%%%
d = 1; % 1,2,3,4,5,6;
%%%%%%%%%%%%%%%%%%%%(Change this to obtain different curves)%%%%%%%%%%%%%%%%%%%%%%%
kappan = n^(-1/4); % n^{-1/4}; n^{-1/3}; n^{-2/5}; Tuning parameter
delta = 0:0.02:0.2;
rept = 10000; %10000 Monte Carlo replications
B = 500; %500 bootstrap replications
lambdasum_hat = zeros(rept,1); % storage for sum of square of the k-r smallest singular values
WLL = zeros(B,3); % Storage for realization of the limit by nonparametric bootstrap
rej = zeros(rept,3); % indicator for rejection
asysize = zeros(length(delta),3);
% the first one for structural estimator of the derivative
% the second and thrrid ones for numerical estimator of the derivative with slightly different formula  
for j=1:length(delta) 
    for i = 1:rept
        rng(i^2+5)
       %% DGPs
        m = 6; 
        k = 6;
        Pi = diag([ones(k-d,1);zeros(d,1)])+delta(j)*eye(k);
        u = randn(n,k); % normal error terms
        % u = sqrt(2)*trnd(4,n,k); %fat tail error terms
        W = randn(n,m);
        % Sigma = kron(ones(1,k),W(:,1)); % heteroskedasticity
        X = W*Pi + u;

    %% Matrix Esetimator
        Pi_hat =W'*X/n; % (pinv(W'*W)*W'*X); m*k
    %% Test statistics
        tempt = sort(svd(Pi_hat).^2);
        [lambdasum_hat(i)] = sum(tempt(1:k-r)); % compute the test statistic for real data
        
    %% Chen and Fang (2019)'s Bootstrap
        for b = 1:B
            rng(i^2+5+10*b)
            ind = unidrnd(n*ones(n,1));
            Wb = W(ind,:);
            Xb = X(ind,:);
            [WLL(b,1)] = TestingRankBootStruct(Wb,Xb,Pi_hat,kappan,r); % Chen and Fang (2019)'s Structural method
            [WLL(b,2),WLL(b,3)] = TestingRankBootNumerical(Wb,Xb,Pi_hat,kappan,r); % Chen and Fang (2019)'s Numerical method
        end
        rej(i,1:3) = [n*lambdasum_hat(i)>quantile(WLL(:,1),0.95),n*lambdasum_hat(i)>quantile(WLL(:,2),0.95),n*lambdasum_hat(i)>quantile(WLL(:,3),0.95)];
    end
    asysize(j,:) = mean(rej);
end
save result1.mat asysize

    