% Figure 3: Comparisons with the KP and the KP-M tests. The number of Monte
% Carlo simulations is 10,000, ...
% In "Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% vec(Pi) follows Omega^(1/2)*N(0+Mu,I), i.i.d.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Testing H_{0}: Rank(Pi_{0})<=1 v.s. H_{1}: Rank(Pi_{0})=2
% To show the overjection of KP, underejection and conservativeness of the
% multiple KP, and improvement of our tests
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n = 1000; % Sample size
rept = 10000; % 10000 Monte Carlo replications
B = 1000; % 1000 Bootstrap replications
rej = zeros(rept,5); % indicator for rejection
CFT = zeros(B,1);
CFA = zeros(B,1);
CFN = zeros(B,1);
Omega1 = eye(4);
Omega2 = [1,0,0,-0.9*sqrt(5);
      0,1,0.9*sqrt(5),0;
      0,0.9*sqrt(5),5,0;
      -0.9*sqrt(5),0,0,5];
RankKP = zeros(2,1);
PV = ones(3,1);
alphaa = 0.05;
%%%%%%%%%%%%%%%%%%%%(Change this to obtain different curves)%%%%%%%%%%%%%%%%%%%%%%%
kappan = n^(-1/4); % n^{-1/4}; n^{-1/3}; n^{-2/5}; Tuning parameter
betaa = alphaa/10; % alpha/5;alpha/10;alpha/15;alpha/20;alpha/25;alpha/30 Tuning parameter
delta = 0;
Omega = Omega1;
Mu = delta*reshape(eye(2),4,1);
temp = chol(Omega)';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
for i = 1:rept
    i
    rng(i^2+5)
    %% DGPs
    datanoise = randn(n,4) + kron(ones(n,1),Mu');
    PI = (temp*datanoise')';
    tempt0 = mean(PI);
    Pi_hat = reshape(tempt0,2,2);
    CovPihat = cov(PI);
    [U,S,V] = svd(Pi_hat);
    tempt1 = kron(V(:,1:end),U(:,1:end));
    tempt2 = reshape(S(1:end,1:end),2*2,1);
    RankKP(1) = n*tempt2'*pinv(tempt1'*CovPihat*tempt1)*tempt2;
    tempt12 = kron(V(:,2:end),U(:,2:end));
    tempt22 = reshape(S(2:end,2:end),1,1);
    RankKP(2) = n*tempt22'*pinv(tempt12'*CovPihat*tempt12)*tempt22;
    PV1 = [1-cdf('chi2',RankKP(1),4);1-cdf('chi2',RankKP(2),1)];
    %% Kleibergen and Paap (2006)'s Direct Method
    rej(i,1) = (PV1(2)<=alphaa);
    %% the Multiple method of Kleibergen and Paap (2006)
    rej(i,2) = prod(PV1<=alphaa);
    PV(1:2) = PV1;
    %% Chen and Fang (2019)'s tests
    rzero = find(PV>betaa, 1)-1;
    if rzero >1
        rej(i,3) = 1;
    elseif rzero <=1
        AT = U(:,rzero+1:end);
        BT = V(:,rzero+1:end);
    else
    end
    tempt3 = sort(svd(Pi_hat).^2);
    RankCF = tempt3(1); % compute the test statistic
    Pi_hatNN = reshape(tempt0./sqrt(diag(CovPihat)'),2,2);
    tempt33 = sort(svd(Pi_hatNN).^2);
    r_hat = min(1,sum(tempt33>kappan^2));
    AA = U(:,r_hat+1:end);
    BA = V(:,r_hat+1:end);
    for b = 1:B
        rng(i^2+5+10*b)
        ind = unidrnd(n*ones(n,1));
        PIstar = PI(ind,:);
        tempt0star = mean(PIstar);
        Pi_hatstar = reshape(tempt0star,2,2);
        Gstar = sqrt(n)*(Pi_hatstar - Pi_hat);
        if rzero <=1
           temp4 = AT'*Gstar*BT;
           temp5 = sort(svd(temp4).^2);
           CFT(b) = temp5(1);
        end
        tempt4 = AA'*Gstar*BA;
        tempt5 = sort(svd(tempt4).^2);
        CFA(b) = tempt5(1);
        tempt6 = sort(svd(Pi_hat+kappan*Gstar).^2);
        CFN(b) = (tempt6(1) - RankCF)/(kappan^2);
    end
    %% CFT
    if rzero <=1
        rej(i,3) = (n*RankCF>quantile(CFT,1-alphaa+betaa));
    end
    %% CFA
    rej(i,4) = (n*RankCF>quantile(CFA,1-alphaa));
    %% CFN
    rej(i,5) = (n*RankCF>quantile(CFN,1-alphaa));
end
asysize = mean(rej);
save result1.mat asysize

    