% Table B.2: The p-values for different tests
% In "Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Loading data 
load SDF.mat
%% Setup
[T,m] = size(R);
k = size(F,2);
%%%%%%%%%%%%%%%%%%%%(Change this to obtain different results)%%%%%%%%%%%%%%%%%%%%%%%
kappan = T^(-1/5); %T^{-1/4}; T^{-1/3}; T^{-2/5}; Tuning parameter
r = 3;
B = 1000; %1000 bootstrap replications
WLL = zeros(B,3); % Storage for realization of the limit by block bootstrap
%% Kleibergen and Paap (2006)'s Method 
RankKP = zeros(r+1,1);
CV = zeros(r+1,1);
PV = zeros(r+1,1);
for q=1:r+1
   [RankKP(q)] = TestingRankKPDDGP3(F,R,q-1);
   CV(q) = icdf('chi2',0.95,(m-q+1)*(k-q+1));
   PV(q) = 1-cdf('chi2',RankKP(q),(m-q+1)*(k-q+1));
end
rejKP = prod(RankKP>CV);
pv = max(PV);
%% Compute test statistic 
Pi_hat = (pinv(F'*F)*F'*R)';
tempt = sort(svd(Pi_hat).^2);
lambdasum_hat = sum(tempt(1:k-r)); % compute the test statistic for real data
%% Chen and Fang (2019)'s Bootstrap
for b = 1:B % block size =1
    rng(5^2+5+10*b)
    ind = unidrnd(T*ones(T,1));
    Fb = F(ind,:);
    Rb = R(ind,:);
    [WLL(b,1)] = TestingRankBootStructDGP3(Fb,Rb,Pi_hat,kappan,r); % Chen and Fang (2019)'s Structural method
    [WLL(b,2),WLL(b,3)] = TestingRankBootNumericalDGP3(Fb,Rb,Pi_hat,kappan,r); % Chen and Fang (2019)'s Numerical method
end
pv0 = mean(T*lambdasum_hat>=WLL);
for b = 1:B % block size =2
    rng(5^2+5+10*b)
    ind1 = unidrnd((T-1)*ones(T/2,1));
    ind = zeros(T,1);
    ind(1:2:end-1) = ind1;
    ind(2:2:end) = ind1+1;
    Fb = F(ind,:);
    Rb = R(ind,:);
    [WLL(b,1)] = TestingRankBootStructDGP3(Fb,Rb,Pi_hat,kappan,r); % Chen and Fang (2019)'s Structural method
    [WLL(b,2),WLL(b,3)] = TestingRankBootNumericalDGP3(Fb,Rb,Pi_hat,kappan,r); % Chen and Fang (2019)'s Numerical method
end
pv1 = mean(T*lambdasum_hat>=WLL);
for b = 1:B % block size =3
    rng(5^2+5+10*b)
    ind1 = unidrnd((T-2)*ones(T/3,1));
    ind = zeros(T,1);
    ind(1:3:end-2) = ind1;
    ind(2:3:end-1) = ind1+1;
    ind(3:3:end) = ind1+2;
    Fb = F(ind,:);
    Rb = R(ind,:);
    [WLL(b,1)] = TestingRankBootStructDGP3(Fb,Rb,Pi_hat,kappan,r); % Chen and Fang (2019)'s Structural method
    [WLL(b,2),WLL(b,3)] = TestingRankBootNumericalDGP3(Fb,Rb,Pi_hat,kappan,r); % Chen and Fang (2019)'s Numerical method
end
pv2 = mean(T*lambdasum_hat>=WLL);
for b = 1:B % block size =4
    rng(5^2+5+10*b)
    ind1 = unidrnd((T-3)*ones((T+2)/4,1));
    ind = zeros(T+2,1);
    ind(1:4:end-3) = ind1;
    ind(2:4:end-2) = ind1+1;
    ind(3:4:end-1) = ind1+2;
    ind(4:4:end) = ind1+3;
    ind = ind(1:T);
    Fb = F(ind,:);
    Rb = R(ind,:);
    [WLL(b,1)] = TestingRankBootStructDGP3(Fb,Rb,Pi_hat,kappan,r); % Chen and Fang (2019)'s Structural method
    [WLL(b,2),WLL(b,3)] = TestingRankBootNumericalDGP3(Fb,Rb,Pi_hat,kappan,r); % Chen and Fang (2019)'s Numerical method
end
pv3 = mean(T*lambdasum_hat>=WLL);