% Tables2/3/4/B1: Rejection rates of rank tests for the model at alpha = 5%
% CF-T
% In "Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DGPs 1 and 2
% X_{i} = W_{i}*Pi_{0} + u_{i}
% X_{i} is 1*k, W_{i} is 1*m and U_{i} is 1*k
% Pi is m*k, m>=k
% DGP 3, see Kleibergen and Paap (2006).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Testing H_{0}: Rank(Pi_{0})<=r v.s. H_{1}: Rank(Pi_{0})>r
% General Finite Sample Examination
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%(Change this to obtain different results)%%%%%%%%%%%%%%%%%%%%%%%
DGP = 1; % DGP1 and DGP2: completely simulated data (same); DGP3: real data based simulated data
delta =(0:0.1:1)*(DGP==1)+(0:0.1:1)*(DGP==2)+(0:0.01:0.1)*(DGP==3); % The strength of H_{1}
%%%%%%%%%%%%%%%%%%%%(Change this to obtain different results)%%%%%%%%%%%%%%%%%%%%%%%
n = 50; %50; 100; 300; 1000; %sample size
n = n*(DGP==1) + n*(DGP==2) + 330*(DGP==3);
r = 2*(DGP==1) + 3*(DGP==2) + 3*(DGP==3);
rept = 5000; %5000 Monte Carlo replications
B = 500; %500 bootstrap replications
lambdasum_hat = zeros(rept,1); % storage for sum of square of the k-r smallest singular values
WLL = zeros(B,1); % Storage for realization of the limit by nonparametric bootstrap
rej = zeros(rept,1); % indicator for rejection
% the two-step method for structural estimator of the derivative
asysize = zeros(length(delta),1);
load SDF.mat
Pi_1 = (pinv(F'*F)*F'*R)';
VH = corr(F(:,2:4));
alphaa = 0.05;
%%%%%%%%%%%%%%%%%%%%(Change this to obtain different results)%%%%%%%%%%%%%%%%%%%%%%%
betaa = alphaa/10; % alpha/5;alpha/10;alpha/15;alpha/20;alpha/25;alpha/30 Tuning parameter
for j=1:length(delta)
    j
    for i = 1:rept
        rng(i^2+5)
    %% DGPs
        if DGP==1 
            %% Parameters Setup
            m = 4; 
            k = 4;
            Pi = diag([ones(k-2,1);zeros(2,1)])+delta(j)*eye(k);
            v = randn(n+1,k); % normal error terms
            % v = sqrt(2)*trnd(4,n+1,k); %fat tail error terms
            vg = ((ones(k,k)/k)*(v'))';
            u = v(2:end,:)-vg(1:end-1,:);
            W = randn(n,m);
            Sigma = kron(ones(1,k),W(:,1));
            X = W*Pi + Sigma.*u;
        elseif DGP==2 
            %% Parameters Setup
            m = 4; 
            k = 4;
            Pi = diag([ones(k-2,1);zeros(2,1)])+delta(j)*eye(k);
            v = randn(n+1,k); % normal error terms
            % v = sqrt(2)*trnd(4,n+1,k); %fat tail error terms
            vg = ((ones(k,k)/k)*(v'))';
            u = v(2:end,:)-vg(1:end-1,:);
            W = randn(n,m);
            Sigma = kron(ones(1,k),W(:,1));
            X = W*Pi + Sigma.*u;
        elseif DGP==3
            %% Parameters Setup
            m = 10; 
            k = 4;
            Sigma = load('Sigma.txt');
            Gamma = load('Gamma.txt');
            alpha = [0.0813,-0.0271,-0.6203,-0.0460];
            beta = [-0.3411;-0.1277;-0.3838;-0.5312;-0.2728;-0.3527;-0.2188;-0.2934;-0.2035;-0.3427];
            Pi = beta*alpha+delta(j)*(Pi_1-beta*alpha);
            v = mvnrnd(zeros(1,m),Sigma/100,n+1); % normal error terms
            vg = (Gamma*(v'))';
            u = v(2:end,:)+vg(1:end-1,:);
            F = [ones(n,1),mvnrnd(zeros(1,k-1),VH,n)];
            R = F*(Pi') + u;
            X = R;
            W = F;
        else
        end

        if DGP==1 || DGP==2   
        %% Matrix Esetimator
            Pi_hat =W'*X/n; % (pinv(W'*W)*W'*X); m*k
        %% Test statistics
            tempt = sort(svd(Pi_hat).^2);
            [lambdasum_hat(i)] = sum(tempt(1:k-r)); % compute the test statistic for real data
        %% Chen and Fang (2019)'s Bootstrap (two step procedure)
            RankKP = zeros(r+1,1);
            PV = zeros(r+2,1);
            PV(r+2) = 1;
            for q=1:r+1
                [RankKP(q)] = TestingRankKPD(W,X,q-1);
                PV(q) = 1-cdf('chi2',RankKP(q),(m-q+1)*(k-q+1));
            end
            rzero = find(PV>betaa, 1)-1;
            if rzero >r
                rej(i,1) = 1;
            elseif rzero <= r
                for b = 1:B
                    rng(i^2+5+10*b)
                    ind1 = unidrnd((n-1)*ones(n/2,1));
                    ind = zeros(n,1);
                    ind(1:2:end-1) = ind1;
                    ind(2:2:end) = ind1+1;
                    Wb = W(ind,:);
                    Xb = X(ind,:);
                    [WLL(b,1)] = TestingRankBootStructTwo(Wb,Xb,Pi_hat,rzero,r); % Chen and Fang (2019)'s Structural method (two step procedure)
                end
                rej(i,1) = [n*lambdasum_hat(i)>quantile(WLL(:,1),1-alphaa+betaa)];
            end
        elseif DGP==3
        %% Matrix Esetimator
            Pi_hat = (pinv(W'*W)*W'*X)';
        %% Test statistics
            tempt = sort(svd(Pi_hat).^2);
            [lambdasum_hat(i)] = sum(tempt(1:k-r)); % compute the test statistic for real data
        %% Chen and Fang (2019)'s Bootstrap
            RankKP = zeros(r+1,1);
            PV = zeros(r+2,1);
            PV(r+2) = 1;
            for q=1:r+1
                [RankKP(q)] = TestingRankKPDDGP3(W,X,q-1);
                PV(q) = 1-cdf('chi2',RankKP(q),(m-q+1)*(k-q+1));
            end
            rzero = find(PV>betaa, 1)-1;
            if rzero >r
                rej(i,1) = 1;
            elseif rzero <= r
                for b = 1:B
                    rng(i^2+5+10*b)
                    ind1 = unidrnd((n-1)*ones(n/2,1));
                    ind = zeros(n,1);
                    ind(1:2:end-1) = ind1;
                    ind(2:2:end) = ind1+1;
                    Wb = W(ind,:);
                    Xb = X(ind,:);
                    [WLL(b,1)] = TestingRankBootStructDGP3Two(Wb,Xb,Pi_hat,rzero,r); % Chen and Fang (2019)'s Structural method (two step procedure)
                end
                rej(i,1) = [n*lambdasum_hat(i)>quantile(WLL(:,1),1-alphaa+betaa)];
            end
        else
        end
    end
    asysize(j,:) = mean(rej);
end

save result1.mat lambdasum_hat asysize

    