function [WLL2,WLL22] = TestingRankBootNumericalDGP3(Wb,Xb,Pi_hat,kappan,r)
% Calculate the realization value of the limting distribution by numerical bootstrap: DGP3
% In "Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs
% Wb: n*k
% Xb: n*m
% Pi_hat: m*k
% kappan: tuning parameter for numerical derivative estimator
% r: the hypothesized rank
% Outputs
% WLL2: the bootstrapped limit by using numerical estimator for derivatives
% WLL22: the bootstrapped limit by using numerical estimator for derivatives (zeros value of phi)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
warning off all
opts = optimset('Display','off');
[n,k] = size(Wb);
Pi_hatb = (pinv(Wb'*Wb)*Wb'*Xb)';
M = sqrt(n)*(Pi_hatb-Pi_hat);
%% Numerical method for estimating the derivative
tempt1 = sort(svd(Pi_hat+kappan*M).^2);
tempt2 = sort(svd(Pi_hat).^2);
WLL2 = (sum(tempt1(1:k-r))-sum(tempt2(1:k-r)))/(kappan^2); % numerical estimation of realizations
WLL22 = sum(tempt1(1:k-r))/(kappan^2); % (zeros value of phi)
end