function [WLL1] = TestingRankBootStruct(Wb,Xb,Pi_hat,kappan,r)
% Calculate the realization value of the limting distribution by structural bootstrap
% In "Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs
% Wb: n*m
% Xb: n*k
% Pi_hat: m*k
% kappan: tuning parameter for numerical derivative estimator
% r: the hypothesized rank
% Outputs
% WLL1: the bootstrapped limit by using structual estimator for derivatives
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
warning off all
opts = optimset('Display','off');
[n,k] = size(Xb);
Pi_hatb = Wb'*Xb/n; % pinv(Wb'*Wb)*Wb'*Xb; m*k
M = sqrt(n)*(Pi_hatb-Pi_hat);
%% Structural method for estimating the derivative
[U,~,V] = svd(Pi_hat); 
S = svd(Pi_hat); 
r_hat = min(r,sum(S>kappan));
A = U(:,r_hat+1:end);
B = V(:,r_hat+1:end);
tempt1 = A'*M*B;
tempt2 = sort(svd(tempt1).^2);
WLL1 = sum(tempt2(1:k-r));
end