function [WLL1] = TestingRankBootStructDGP3Two(Wb,Xb,Pi_hat,rzero,r)
% Calculate the realization value of the limting distribution by structural bootstrap
% For Two step procedure
% In "Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs
% Wb: n*k
% Xb: n*m
% Pi_hat: m*k
% rzero: the true rank
% r: the hypothesized rank
% Outputs
% WLL1: the bootstrapped limit by using structual estimator for derivatives
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
warning off all
opts = optimset('Display','off');
[n,k] = size(Wb);
Pi_hatb = (pinv(Wb'*Wb)*Wb'*Xb)';
M = sqrt(n)*(Pi_hatb-Pi_hat);
%% Structural method for estimating the derivative
[U,~,V] = svd(Pi_hat); 
A = U(:,rzero+1:end);
B = V(:,rzero+1:end);
tempt1 = A'*M*B;
tempt2 = sort(svd(tempt1).^2);
WLL1 = sum(tempt2(1:k-r));
end