function [RankKP] = TestingRankKP(W,X,r)
% Calculate the test statistic in Kleibergen and Paap (2006) with independent data
% In "Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs
% W: n*m
% X: n*k
% r: the hypothesized rank
% Outputs
% RankPK: the test statistic in Kleibergen and Paap (2006) with independent data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
warning off all
opts = optimset('Display','off');
[n,k] = size(X);
m = size(W,2);
Pi_hat = W'*X/n; % pinv(Wb'*Wb)*Wb'*Xb; m*k
CovEst = zeros(m*k,m*k);
for i=1:n
    CovEst = CovEst + reshape(W(i,:)'*X(i,:)-Pi_hat,m*k,1)*(reshape(W(i,:)'*X(i,:)-Pi_hat,m*k,1))'/n;
end
[U,S,V] = svd(Pi_hat); 
tempt1 = kron(V(:,r+1:end),U(:,r+1:end));
tempt2 = reshape(S(r+1:end,r+1:end),(m-r)*(k-r),1);
RankKP = n*tempt2'*pinv(tempt1'*CovEst*tempt1+1/n)*tempt2;
end