function [RankKP] = TestingRankKPDDGP3(W,X,r)
% Calculate the test statistic in Kleibergen and Paap (2006) with AR(1) dependent data and DGP3
% In "Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019).
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs
% W: n*k
% X: n*m
% r: the hypothesized rank
% Outputs
% RankPK: the test statistic in Kleibergen and Paap (2006) with AR(1)
% dependent data and DGP3
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
warning off all
opts = optimset('Display','off');
[n,k] = size(W);
m = size(X,2);
Pi_hat = (pinv(W'*W)*W'*X)';
CovEst1 = zeros(m*k,m*k);
for i=1:n
    CovEst1 = CovEst1 + reshape((pinv(W'*W/n)*W(i,:)'*X(i,:))'-Pi_hat,m*k,1)*(reshape((pinv(W'*W/n)*W(i,:)'*X(i,:))'-Pi_hat,m*k,1))'/n;
end
CovEst2 = zeros(m*k,m*k);
for i=1:n-1
    CovEst2 = CovEst2 + reshape((pinv(W'*W/n)*W(i,:)'*X(i,:))'-Pi_hat,m*k,1)*(reshape((pinv(W'*W/n)*W(i+1,:)'*X(i+1,:))'-Pi_hat,m*k,1))'/(n-1);
end
CovEst = CovEst1 + CovEst2 + CovEst2';
[U,S,V] = svd(Pi_hat); 
tempt1 = kron(V(:,r+1:end),U(:,r+1:end));
tempt2 = reshape(S(r+1:end,r+1:end),(m-r)*(k-r),1);
RankKP = n*tempt2'*pinv(tempt1'*CovEst*tempt1+1/n)*tempt2;
end