function [RankKP] = TestingRankKPLimit(VM,n)
% Simulate the limting distribution of the test statistic in Kleibergen and Paap (2006)
% Special case: the true value of matrix is 2*2 zero, Gaussian limit and r=1
% In "Inference on the Rank of a Matrix" by Qihui Chen and Zheng Fang (2019)
% Edited by Qihui Chen and Zheng Fang (2019)
% Updated on 5/1/2019
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs
% VM: 4*4 the variance matrix of the Gaussian limiting distribution
% n: the number of replications
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Outputs
% RankPK: n*1 the realization of the limiting distribution of the test statistic


RankKP = zeros(n,1);
Data = mvnrnd(zeros(4,1),VM,n);
for i=1:n
   M = reshape(Data(i,:),2,2); 
   [U,S,V] = svd(M);
   tempt = kron(V(:,2),U(:,2));
   RankKP(i) = S(2,2)^2/(tempt'*VM*tempt);
end